function clusterSpotsTrackedFromFrame(aImarisApplicationID,maxgroup,dist,timesDist)
%
%
%   <CustomTools>
%      <Menu>
%       <Submenu name="Spots Functions">
%        <Item name="Cluster Spots Tracked From Frame" icon="Matlab" tooltip="Cluster Spots in Groups">
%          <Command>MatlabXT::clusterSpotsTrackedFromFrame(%i)</Command>
%        </Item>
%       </Submenu>
%      </Menu>
%    </CustomTools>
%
% Description : The function creates cluster Spots objects within a distance 
% smaller than "dist" selected by the user from a Spots selection.
% The function prompts the average minimal distance between spots from the
% first time point to help the user enter a good distance range
% (generally 2 times or more).
% The user can also select a maximum number of clusters to produce or put
% "inf" to retrieve all possible clusters.
% The user selects the Key Frame at which the clustering is done. Tracks 
% belonging to this key frame are kept for previous and following
% frames.
% The function attributes the maximum number of spots to the first cluster
% and so on.
% Clusters show the average number of spots per time points in parenthesis
% beside their name.
% When the function is used outside of the Imaris menu, set "maxgroup" and 
% "dist" to the desired values and timesDist to "1".
% If you need to automatically calculate the distance, set "dist = -1" and
% "timesDist" to the number of times the average minimal value.
%
% Created by Dominic Filion, February 2016
% Responsable, Plateau de microscopie
% Institut de Recherches Cliniques de Montral
% 110 avenue des Pins
% Montral (Qubec)
% H2W 1R7
% dominic.filion@ircm.qc.ca


% Set up connection between Imaris and MATLAB
if isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
    vImarisApplication = aImarisApplicationID;
else
    % connect to Imaris interface
    javaaddpath ImarisLib.jar
    vImarisLib = ImarisLib;
    if ischar(aImarisApplicationID)
        aImarisApplicationID = round(str2double(aImarisApplicationID));
    end
    vImarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
end

% Check if there is a scene created
vSurpassScene = vImarisApplication.GetSurpassScene;
if isequal(vSurpassScene, [])
    msgbox('Please create some Spots in the Surpass scene!');
    return;
end

% Get the selection and make sure that it is a Spots item
vSpots = vImarisApplication.GetFactory.ToSpots(vImarisApplication.GetSurpassSelection);
if ~vImarisApplication.GetFactory.IsSpots(vSpots)
    msgbox('Please select a Spots item!');
    return;
end

% Get the spot coordinates, time indices and radius
spotValues = vSpots.Get();
coords = spotValues.mPositionsXYZ;
aTimes = vSpots.GetIndicesT + 1;
radii = spotValues.mRadii;
% Check if the spots have been tracked
vEdges = vSpots.GetTrackEdges + 1;

% Get the color of the present spots group and create contrasted colors
vRGBA = vSpots.GetColorRGBA;
[R,G,B] = rgba2rgb(vRGBA);
color(1) = rgb2rgba(B,R,G);
color(2) = rgb2rgba(G,B,R);
color(3) = rgb2rgba(255-R,255-G,255-B);
color(4) = rgb2rgba(255-G,255-B,255-R);
color(5) = rgb2rgba(255-B,255-R,255-G);

% initialize values
% Ask the user to enter a maximum number of groups
if nargin <= 1
    maxgroup = inputdlg({'Enter max number of clusters (inf = all clusters):'},'Input',1,{'2'});
    if isempty(maxgroup)
        return
    end
    maxgroup = str2double(maxgroup{1});
end

if maxgroup < 1
    msgbox('The max cluster number must be greater than 1');
    return
end

grInd = nan(size(aTimes,1),1);

if nargin <= 2
    % Prompt the distance for the current time point
    ind = aTimes == vImarisApplication.GetVisibleIndexT + 1;
    xyz = coords(ind,:);
    [~,dist] = clusterXY(xyz,0,2);
end

if nargin <= 3
    timesDist = 2.5;
end

if ~isempty(vEdges)
    vEP = nan(size(aTimes,1),1);
    vEN = nan(size(aTimes,1),1);
    vEP(vEdges(:,2)) = vEdges(:,1);
    vEN(vEdges(:,1)) = vEdges(:,2);
end

    % select each time frame
    % Ask the user to enter a the key Frame number
    if nargin <= 1
        keyFrame = inputdlg({'Enter the key Frame number'},'Input',1,{'6'});
        if isempty(keyFrame)
            return
        end
        keyFrame = str2double(keyFrame{1});
    end
    ind = aTimes==keyFrame;
    
    % select the coordinate of spots in that frame
    xyz = coords(ind,:);
    
    % attribute an index relative to a group within distance "dist"
    % prompt the user at the first run since "dist==0"
    [groupIND,dist] = clusterXY(xyz,dist,timesDist);
    if isempty(dist)
        close(hw)
        return
    end
    indg = maingroups(groupIND);
    grInd(ind) = indg;

    % attributed the index value the the previous points
    for j = keyFrame:-1:1
        ind = aTimes == j;
        vEPs = vEP(ind);
        objSelected = ~isnan(vEPs);
        vEPs = vEPs(objSelected);
        grInds = grInd(ind);
        grInd(vEPs) = grInds(objSelected);
    end
    
    % attributed the index value the the previous points
    for j = keyFrame:max(aTimes)
        ind = aTimes == j;
        vENs = vEN(ind);
        objSelected = ~isnan(vENs);
        vENs = vENs(objSelected);
        grInds = grInd(ind);
        grInd(vENs) = grInds(objSelected);
    end


% create new groups
vSpotsGroup = vImarisApplication.GetFactory.CreateDataContainer;
vSpotsGroup.SetName(sprintf('%s clusters [< %.2f]', char(vSpots.GetName), dist));

for j = 1:min(max(grInd),maxgroup)
    vSpotsXYZ = coords(grInd==j,:);
    vSpotsTime = aTimes(grInd==j) - 1;
    vRadii = radii(grInd==j);
    vNewSpots = vImarisApplication.GetFactory.CreateSpots;
    vNewSpots.Set(vSpotsXYZ, vSpotsTime, vRadii);
    
    % get the tracks associated with the group
    if ~isempty(vEdges)
        indnan = isnan(vEN);
        newIndexNumbers = 1:sum(grInd==j);
        newIndex = zeros(size(grInd));
        newIndex(grInd==j) = newIndexNumbers;
        vNewEdges = [newIndex(grInd==j&~indnan) newIndex(vEN(grInd==j&~indnan))];
        vNewEdges = int32(vNewEdges-1);
        vNewSpots.SetTrackEdges(vNewEdges);
    end
    
    % Give a color contrasted with the original color
    if j <= 5
        vNewSpots.SetColorRGBA(color(:,j));
    else
        vNewSpots.SetColorRGBA(floor((256^3)*rand));
    end
    % set the cluster name and write the average spots per timepoint
    vNewSpots.SetName(sprintf('Cluster %d (%.1f)',...
        j,sum(double(grInd==j))/double(max(aTimes))));
    vSpotsGroup.AddChild(vNewSpots, -1);   
end

% add the spots group
if maxgroup == 1
    vNewSpots.SetName(sprintf('%s subgroup',char(vSpots.GetName)));
    vSurpassScene.AddChild(vNewSpots, -1);
else
    vSurpassScene.AddChild(vSpotsGroup, -1);
end


%% Function to cluster groups within a distance "dist" to each other

    function [groupIND,dist] = clusterXY(xy,dist,timesDist)
        
        D = newpdist(xy);
        if ~isempty(D)
            % Calculate the average minimal distance between spots of the
            % first frame
            if dist <= 0
                di = logical(diag(ones(size(D,1),1)));
                D(di) = nan;
                distmin = mean(min(D));
                D(di) = 0;
                if dist == 0
                    dist = inputdlg({'Enter max distance (actual value is average distance):'},...
                    'Input',1,{num2str(distmin*timesDist)});
                else
                    dist = {num2str(timesDist*distmin)};
                end
                if isempty(dist)
                    groupIND = zeros(size(D,1),1);
                    return
                end
                dist = str2double(dist{1});
            end
            groupIND = zeros(size(D,1),1);
            list_ingroup = false(size(xy,1),1);
            list_done = false(size(xy,1),1);
            groupnum = 1;
            while ~all(list_done)
                todo = find(~list_done);
                todo = todo(1);
                currentgroupnum = groupnum;
                cindex = [];
                while groupnum == currentgroupnum
                    index = find(D(todo,:) < dist);
                    cindex = checkIndex(cindex,index);
                    list_ingroup(index) = 1;
                    list_done(todo) = 1;
                    islast = find(cindex == todo);
                    if islast == length(cindex)
                        groupnum = groupnum + 1;
                    else
                        todo = cindex(islast + 1);
                    end
                end
                for ii = 1:size(cindex,2)
                    groupIND(cindex(ii)) = currentgroupnum;
                end
            end
        else
            groupIND = [];
        end
    end
%--------------------------------------------------------------------------
    function grInd = maingroups(groupIND)
        
        groupValues = unique(groupIND);
        occurences = zeros(size(groupValues,1),1);
        for ii = 1:size(groupValues,1)
            occurences(ii) = sum(groupIND==groupValues(ii));
        end
        [~,grInd2] = sort(occurences,'descend');
        grInd = zeros(size(groupIND,1),1);
        for ii = 1:size(groupValues,1)
            grInd(groupIND==groupValues(grInd2(ii))) = groupValues(ii);
        end
        
    end
%--------------------------------------------------------------------------
% check if index is part of the list and update list
    function newind = checkIndex(oldind,ind)
        
        isin = false(length(ind),1);
        for ii = 1:length(ind)
            isin(ii) = any(oldind == ind(ii));
        end
        newind = [oldind ind(~isin)];
    end
%--------------------------------------------------------------------------
    function D = newpdist(xy)
        
        D = zeros(size(xy,1));
        for ii = 1:size(xy,1)
            p1 = repmat(xy(ii,:),[size(xy,1) 1]);
            D(ii,:) = sqrt(sum((xy - p1).^2,2));
        end
    end
%--------------------------------------------------------------------------
    function [R,G,B,A] = rgba2rgb(rgba)
        
        if rgba < 0
            rgba = rgba + 2^32;
        end
        str = dec2bin(rgba);
        strz = sprintf('%032d',0);
        strz(end-length(str)+1:end) = str;
        A = bin2dec(strz(1:8));
        B = bin2dec(strz(9:16));
        G = bin2dec(strz(17:24));
        R = bin2dec(strz(25:32));
    end
%--------------------------------------------------------------------------
    function rgba = rgb2rgba(R,G,B)
        
        rgba = bin2dec(sprintf('%08d%08d%08d',str2double(dec2bin(B)),...
            str2double(dec2bin(G)),str2double(dec2bin(R))));
    end
end
