function createGroupsFromLine(aImarisApplicationID)
% Export spots to matlab
%
%    <CustomTools>
%      <Menu>
%       <Submenu name="ANALYZE WAVE">
%        <Item name="Create Groups From Line" icon="Matlab" tooltip="CreateGroupsFromLine">
%          <Command>MatlabXT::createGroupsFromLine(%i)</Command>
%        </Item>
%       </Submenu>
%      </Menu>
%    </CustomTools>
%
% Copyright (c) 2016, Dominic Filion

% Set up connection between Imaris and MATLAB
if isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
    vImarisApplication = aImarisApplicationID;
else
    % connect to Imaris interface
    javaaddpath ImarisLib.jar
    vImarisLib = ImarisLib;
    if ischar(aImarisApplicationID)
        aImarisApplicationID = round(str2double(aImarisApplicationID));
    end
    vImarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
end

% Check if there is a scene created
vSurpassScene = vImarisApplication.GetSurpassScene;
if isequal(vSurpassScene, [])
    msgbox('Please create some Spots in the Surpass scene!');
    return;
end

% Get the currently selected object in the surpass scene and check that it
% is a Spots object
vSpots = vImarisApplication.GetSurpassSelection();
vSpots = vImarisApplication.GetFactory().ToSpots(vSpots);
if isempty(vSpots)
    errordlg('Please select a Spots object!');
    return
end

% Get the spot coordinates, time indices and radius
spotValues = vSpots.Get();
aXYZ = spotValues.mPositionsXYZ;
aTimes = vSpots.GetIndicesT + 1;
radii = spotValues.mRadii;
% Check if the spots have been tracked
vEdges = vSpots.GetTrackEdges + 1;

if ~isempty(vEdges)
    vEP = nan(size(aTimes,1),1);
    vEN = nan(size(aTimes,1),1);
    vEP(vEdges(:,2)) = vEdges(:,1);
    vEN(vEdges(:,1)) = vEdges(:,2);
end


% select each time frame
    % Ask the user to enter a the key Frame number
    if nargin <= 1
        keyFrame = inputdlg({'Enter the key Frame number'},'Input',1,{'6'});
        if isempty(keyFrame)
            return
        end
        keyFrame = str2double(keyFrame{1});
    end
    ind = aTimes==keyFrame;


% select the coordinate of spots in that frame
xyz = aXYZ(ind,:);
plot(xyz(:,1),xyz(:,2),'x');
hl = imline();
pos = getPosition(hl);

p = polyfit(pos(:,2),pos(:,1),1);

% calculate the distance for each points at key Frame
distances = abs(-p(1).*xyz(:,2)+xyz(:,1)-p(2))./sqrt(p(1).^2+1.^2);

cp = cd();
[filename,path] = uiputfile('*.mat','Please enter Filename for distances values');
if path ~= 0
    cd(path)
    save(filename,'distances');
end
cd(cp);

grInd = nan(size(aTimes,1),1);

if nargin <= 1
    numgroup = inputdlg({'Enter the Number of groups'},'Input',1,{'10'});
    if isempty(keyFrame)
        return
    end
    numgroup = str2double(numgroup{1});
end

distances = ceil(distances./max(distances).*numgroup);
grInd(ind) = distances;



% attributed the index value the the previous points
    for j = keyFrame:-1:1
        ind = aTimes == j;
        vEPs = vEP(ind);
        objSelected = ~isnan(vEPs);
        vEPs = vEPs(objSelected);
        grInds = grInd(ind);
        grInd(vEPs) = grInds(objSelected);
    end
    
    % attributed the index value the the previous points
    for j = keyFrame:max(aTimes)
        ind = aTimes == j;
        vENs = vEN(ind);
        objSelected = ~isnan(vENs);
        vENs = vENs(objSelected);
        grInds = grInd(ind);
        grInd(vENs) = grInds(objSelected);
    end


% give a groupId relative to their distance from the line
% Get the color of the present spots group and create contrasted colors
vRGBA = vSpots.GetColorRGBA;
[R,G,B] = rgba2rgb(vRGBA);
color(1) = rgb2rgba(B,R,G);
color(2) = rgb2rgba(G,B,R);
color(3) = rgb2rgba(255-R,255-G,255-B);
color(4) = rgb2rgba(255-G,255-B,255-R);
color(5) = rgb2rgba(255-B,255-R,255-G);

% Create the new spots from Group1
% create new group
vSpotsGroup = vImarisApplication.GetFactory.CreateDataContainer;
vSpotsGroup.SetName(sprintf('%s distances groups', char(vSpots.GetName)));

for j = 1:min(max(grInd),numgroup)
    vSpotsXYZ = aXYZ(grInd==j,:);
    vSpotsTime = aTimes(grInd==j) - 1;
    vRadii = radii(grInd==j);
    vNewSpots = vImarisApplication.GetFactory.CreateSpots;
    vNewSpots.Set(vSpotsXYZ, vSpotsTime, vRadii);
    
    % get the tracks associated with the group
    if ~isempty(vEdges)
        indnan = isnan(vEN);
        newIndexNumbers = 1:sum(grInd==j);
        newIndex = zeros(size(grInd));
        newIndex(grInd==j) = newIndexNumbers;
        vNewEdges = [newIndex(grInd==j&~indnan) newIndex(vEN(grInd==j&~indnan))];
        vNewEdges = int32(vNewEdges-1);
        vNewSpots.SetTrackEdges(vNewEdges);
    end
    
    % Give a color contrasted with the original color
    if j <= 5
        vNewSpots.SetColorRGBA(color(:,j));
    else
        vNewSpots.SetColorRGBA(floor((256^3)*rand));
    end
    % set the cluster name and write the average spots per timepoint
    vNewSpots.SetName(sprintf('Cluster %d (%.1f)',...
        j,sum(double(grInd==j))/double(max(aTimes))));
    vSpotsGroup.AddChild(vNewSpots, -1);
end

% add the spots group
if numgroup == 1
    vNewSpots.SetName(sprintf('%s subgroup',char(vSpots.GetName)));
    vSurpassScene.AddChild(vNewSpots, -1);
else
    vSurpassScene.AddChild(vSpotsGroup, -1);
end


%--------------------------------------------------------------------------
    function [R,G,B,A] = rgba2rgb(rgba)
        
        if rgba < 0
            rgba = rgba + 2^32;
        end
        str = dec2bin(rgba);
        strz = sprintf('%032d',0);
        strz(end-length(str)+1:end) = str;
        A = bin2dec(strz(1:8));
        B = bin2dec(strz(9:16));
        G = bin2dec(strz(17:24));
        R = bin2dec(strz(25:32));
    end

%--------------------------------------------------------------------------
    function rgba = rgb2rgba(R,G,B)
        
        rgba = bin2dec(sprintf('%08d%08d%08d',str2double(dec2bin(B)),...
            str2double(dec2bin(G)),str2double(dec2bin(R))));
    end
end